/*
 * Copyright 2011 BitMeister Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jp.bitmeister.asn1.type.builtin;

import jp.bitmeister.asn1.annotation.ASN1BuiltIn;
import jp.bitmeister.asn1.annotation.ASN1Tag;
import jp.bitmeister.asn1.exception.ASN1IllegalArgument;
import jp.bitmeister.asn1.processor.ASN1Visitor;
import jp.bitmeister.asn1.type.ASN1TagClass;
import jp.bitmeister.asn1.type.ASN1TagMode;
import jp.bitmeister.asn1.type.ASN1Type;
import jp.bitmeister.asn1.type.PrimitiveType;
import jp.bitmeister.asn1.type.ValueComparable;

/**
 * Represents ASN.1 'REAL' type.
 * 
 * <p>
 * An instance of this class represents a 'REAL' type data, and contains a
 * {@link java.lang.Double} value.
 * </p>
 * 
 * @author WATANABE, Jun. <jwat at bitmeister.jp>
 */
@ASN1BuiltIn
@ASN1Tag(tagClass = ASN1TagClass.UNIVERSAL, value = 9, tagMode = ASN1TagMode.IMPLICIT)
public class REAL extends PrimitiveType<Double> implements ValueComparable {

	/**
	 * Instantiates an empty {@code REAL}.
	 */
	public REAL() {
	}

	/**
	 * Instantiates a {@code REAL} and initialize it with the {@code Double}
	 * value.
	 * 
	 * @param value
	 *            The value assigned to the instance.
	 */
	public REAL(Double value) {
		set(value);
	}

	/**
	 * Instantiates a {@code REAL} and initialize it with the {@code Float}
	 * value.
	 * 
	 * @param value
	 *            The value assigned to the instance.
	 */
	public REAL(Float value) {
		set(value);
	}

	/**
	 * Instantiates a {@code REAL} and initialize it with the value specified by
	 * the M*B^E formula.
	 * 
	 * @param mantissa
	 *            Mantissa.
	 * @param base
	 *            Base. It can take values 2 or 10.
	 * @param exponent
	 *            Exponent.
	 */
	public REAL(long mantissa, int base, int exponent) {
		set(mantissa, base, exponent);
	}

	/**
	 * Sets the {@code Float} value to the instance.
	 * 
	 * @param value
	 *            The value assigned to the instance.
	 */
	public void set(Float value) {
		set((double) value);
	}

	/**
	 * Sets the value specified by the M*B^E formula to the instance.
	 * 
	 * @param mantissa
	 *            Mantissa.
	 * @param base
	 *            Base. It can take values 2 or 10.
	 * @param exponent
	 *            Exponent.
	 */
	public void set(long mantissa, int base, int exponent) {
		if (base != 2 && base != 10) {
			ASN1IllegalArgument ex = new ASN1IllegalArgument();
			ex.setMessage("Invalid base value '" + base
					+ "'. Base must be '2' or '10'.", null, getClass(), null,
					null);
			throw ex;
		}
		set(mantissa * Math.pow(base, exponent));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * jp.bitmeister.asn1.type.ValueComparable#compareTo(jp.bitmeister.asn1.
	 * type.ASN1Type)
	 */
	@Override
	public int compareTo(ASN1Type other) {
		return value().compareTo(((REAL) other).value());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * jp.bitmeister.asn1.type.ASN1Type#accept(jp.bitmeister.asn1.processor.
	 * ASN1Visitor)
	 */
	@Override
	public <E extends Throwable> void accept(ASN1Visitor<E> visitor) throws E {
		visitor.visit(this);
	}

}
